/* pi.c
 * ----
 * Simple pi calculation targetted to the epiphany, using OMPi.
 *
 * The OMPi team, 2015.
 */

#include <stdio.h>
#include <omp.h>

#define N 2000000      /* Intervals */


/* Naive pi calculation based on the trapezoid rule.
 */
void calc_pi(int nthr)
{
	double pi = 0.0;
	
	/* Executed @ host (zynq, aka device 0) */
	#pragma omp target map(pi)
	{
		/* Offloaded @ default device (normally epiphany). 
		* Note that pi is explicitly mapped as tofrom.
		*/
		#pragma omp parallel num_threads(nthr) reduction(+:pi)
		{
			double W = 4.0 / N, W2 = W*W/16.0;
			int i = omp_get_thread_num();
			
			for ( ; i < N; i += nthr)
				pi += W / (1.0 + (0.5 + i)*(0.5 + i)*W2);
		}
	}
	printf("pi = %.10lf", pi);
}


/* Calculate pi with 1, 2, 4, 8, 16 threads. 
 */
void main()
{
	int    nthr, max_threads;
	double t1, t2;
	
	if(omp_get_default_device() == 0)
	{
		printf("Calculating pi on the Zynq, using 1 and 2 threads..\n");
		max_threads = 2;
	}
	else
	{
		printf("Calculating pi on the Epiphany, using 1, 2, 4, 8, and 16 threads..\n");
		max_threads = 16;
	}
	
	for (nthr = 1; nthr <= max_threads; nthr <<= 1)
	{
		t1 = omp_get_wtime();
		calc_pi(nthr);
		t2 = omp_get_wtime();
		
		printf("\t[%2d threads => %2.5lf sec]\n", nthr, t2-t1);
	}
}

